@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2015 The git2r contributors
+ *  Copyright (C) 2013-2020 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -16,11 +16,12 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "git2.h"
+#include <git2.h>
 
 #include "git2r_arg.h"
 #include "git2r_clone.h"
 #include "git2r_cred.h"
+#include "git2r_deprecated.h"
 #include "git2r_error.h"
 #include "git2r_transfer.h"
 
@@ -33,7 +34,7 @@
  * @return 0
  */
 static int git2r_clone_progress(
-    const git_transfer_progress *progress,
+    const GIT2R_INDEXER_PROGRESS *progress,
     void *payload)
 {
     int kbytes = progress->received_bytes / 1024;
@@ -85,7 +86,7 @@ SEXP git2r_clone(
     SEXP credentials,
     SEXP progress)
 {
-    int err;
+    int error;
     git_repository *repository = NULL;
     git_clone_options clone_opts = GIT_CLONE_OPTIONS_INIT;
     git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
@@ -97,7 +98,7 @@ SEXP git2r_clone(
         git2r_error(__func__, NULL, "'local_path'", git2r_err_string_arg);
     if (git2r_arg_check_logical(bare))
         git2r_error(__func__, NULL, "'bare'", git2r_err_logical_arg);
-    if (branch != R_NilValue && git2r_arg_check_string(branch))
+    if ((!Rf_isNull(branch)) && git2r_arg_check_string(branch))
         git2r_error(__func__, NULL, "'branch'", git2r_err_string_arg);
     if (git2r_arg_check_logical(checkout))
         git2r_error(__func__, NULL, "'checkout'", git2r_err_logical_arg);
@@ -110,7 +111,7 @@ SEXP git2r_clone(
       checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE;
     else
       checkout_opts.checkout_strategy = GIT_CHECKOUT_NONE;
-    
+
     clone_opts.checkout_opts = checkout_opts;
     payload.credentials = credentials;
     clone_opts.fetch_opts.callbacks.payload = &payload;
@@ -119,7 +120,7 @@ SEXP git2r_clone(
     if (LOGICAL(bare)[0])
         clone_opts.bare = 1;
 
-    if (branch != R_NilValue)
+    if (!Rf_isNull(branch))
         clone_opts.checkout_branch = CHAR(STRING_ELT(branch, 0));
 
     if (LOGICAL(progress)[0]) {
@@ -127,18 +128,17 @@ SEXP git2r_clone(
         Rprintf("cloning into '%s'...\n", CHAR(STRING_ELT(local_path, 0)));
     }
 
-    err = git_clone(&repository,
+    error = git_clone(&repository,
                     CHAR(STRING_ELT(url, 0)),
                     CHAR(STRING_ELT(local_path, 0)),
                     &clone_opts);
 
-    if (repository)
-        git_repository_free(repository);
+    git_repository_free(repository);
 
-    if (err)
+    if (error)
         git2r_error(
             __func__,
-            giterr_last(),
+            GIT2R_ERROR_LAST(),
             git2r_err_unable_to_authenticate,
             NULL);
 
