@@ -750,6 +750,9 @@ int git2r_diff_count_file_cb(const git_diff_delta *delta,
 {
     git2r_diff_count_payload *n = payload;
 
+    GIT2R_UNUSED(delta);
+    GIT2R_UNUSED(progress);
+
     n->num_files += 1;
     n->num_hunks = n->num_lines = 0;
     return 0;
@@ -769,6 +772,9 @@ int git2r_diff_count_hunk_cb(const git_diff_delta *delta,
 {
     git2r_diff_count_payload *n = payload;
 
+    GIT2R_UNUSED(delta);
+    GIT2R_UNUSED(hunk);
+
     n->num_hunks += 1;
     if (n->num_hunks > n->max_hunks) { n->max_hunks = n->num_hunks; }
     n->num_lines = 0;
@@ -792,6 +798,10 @@ int git2r_diff_count_line_cb(const git_diff_delta *delta,
 {
     git2r_diff_count_payload *n = payload;
 
+    GIT2R_UNUSED(delta);
+    GIT2R_UNUSED(hunk);
+    GIT2R_UNUSED(line);
+
     n->num_lines += 1;
     if (n->num_lines > n->max_lines)
         n->max_lines = n->num_lines;
@@ -867,6 +877,8 @@ int git2r_diff_get_file_cb(const git_diff_delta *delta,
 {
     git2r_diff_payload *p = (git2r_diff_payload *) payload;
 
+    GIT2R_UNUSED(progress);
+
     /* Save previous hunk's lines in hunk_tmp, we just call the
        hunk callback, with a NULL hunk */
     git2r_diff_get_hunk_cb(delta, /* hunk= */ 0, payload);
@@ -932,6 +944,8 @@ int git2r_diff_get_hunk_cb(const git_diff_delta *delta,
 {
     git2r_diff_payload *p = (git2r_diff_payload *) payload;
 
+    GIT2R_UNUSED(delta);
+
     /* Save previous hunk's lines in hunk_tmp, from the line_tmp
        temporary storage. */
     if (p->hunk_ptr != 0) {
@@ -1013,6 +1027,9 @@ int git2r_diff_get_line_cb(const git_diff_delta *delta,
     char *buffer = short_buffer;
     SEXP line_obj;
 
+    GIT2R_UNUSED(delta);
+    GIT2R_UNUSED(hunk);
+
     PROTECT(line_obj = Rf_mkNamed(VECSXP, git2r_S3_items__git_diff_line));
     Rf_setAttrib(
         line_obj,
