@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2017 The git2r contributors
+ *  Copyright (C) 2013-2019 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -16,15 +16,15 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <Rdefines.h>
-#include "git2.h"
-#include "buffer.h"
+#include <git2.h>
 
 #include "git2r_arg.h"
 #include "git2r_commit.h"
+#include "git2r_deprecated.h"
 #include "git2r_error.h"
 #include "git2r_merge.h"
 #include "git2r_repository.h"
+#include "git2r_S3.h"
 #include "git2r_signature.h"
 
 int git2r_commit_create(
@@ -45,13 +45,11 @@ int git2r_commit_create(
  */
 SEXP git2r_merge_base(SEXP one, SEXP two)
 {
-    int err;
+    int error, nprotect = 0;
     SEXP result = R_NilValue;
-    SEXP repo;
+    SEXP repo_one, repo_two;
     SEXP sha;
-    git_oid oid;
-    git_oid oid_one;
-    git_oid oid_two;
+    git_oid oid, oid_one, oid_two;
     git_commit *commit = NULL;
     git_repository *repository = NULL;
 
@@ -60,47 +58,51 @@ SEXP git2r_merge_base(SEXP one, SEXP two)
     if (git2r_arg_check_commit(two))
         git2r_error(__func__, NULL, "'two'", git2r_err_commit_arg);
 
-    repo = GET_SLOT(one, Rf_install("repo"));
-    repository = git2r_repository_open(repo);
+    repo_one = git2r_get_list_element(one, "repo");
+    repo_two = git2r_get_list_element(two, "repo");
+    if (git2r_arg_check_same_repo(repo_one, repo_two))
+        git2r_error(__func__, NULL, "'one' and 'two' not from same repository", NULL);
+
+    repository = git2r_repository_open(repo_one);
     if (!repository)
         git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
-    sha = GET_SLOT(one, Rf_install("sha"));
-    err = git_oid_fromstr(&oid_one, CHAR(STRING_ELT(sha, 0)));
-    if (err)
+    sha = git2r_get_list_element(one, "sha");
+    error = git_oid_fromstr(&oid_one, CHAR(STRING_ELT(sha, 0)));
+    if (error)
         goto cleanup;
 
-    sha = GET_SLOT(two, Rf_install("sha"));
-    err = git_oid_fromstr(&oid_two, CHAR(STRING_ELT(sha, 0)));
-    if (err)
+    sha = git2r_get_list_element(two, "sha");
+    error = git_oid_fromstr(&oid_two, CHAR(STRING_ELT(sha, 0)));
+    if (error)
         goto cleanup;
 
-    err = git_merge_base(&oid, repository, &oid_one, &oid_two);
-    if (err) {
-        if (GIT_ENOTFOUND == err)
-            err = GIT_OK;
+    error = git_merge_base(&oid, repository, &oid_one, &oid_two);
+    if (error) {
+        if (GIT_ENOTFOUND == error)
+            error = GIT_OK;
         goto cleanup;
     }
 
-    err = git_commit_lookup(&commit, repository, &oid);
-    if (err)
+    error = git_commit_lookup(&commit, repository, &oid);
+    if (error)
         goto cleanup;
 
-    PROTECT(result = NEW_OBJECT(MAKE_CLASS("git_commit")));
-    git2r_commit_init(commit, repo, result);
+    PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_commit));
+    nprotect++;
+    Rf_setAttrib(result, R_ClassSymbol,
+                 Rf_mkString(git2r_S3_class__git_commit));
+    git2r_commit_init(commit, repo_one, result);
 
 cleanup:
-    if (commit)
-        git_commit_free(commit);
+    git_commit_free(commit);
+    git_repository_free(repository);
 
-    if (repository)
-        git_repository_free(repository);
+    if (nprotect)
+        UNPROTECT(nprotect);
 
-    if (!isNull(result))
-        UNPROTECT(1);
-
-    if (err)
-        git2r_error(__func__, giterr_last(), NULL, NULL);
+    if (error)
+        git2r_error(__func__, GIT2R_ERROR_LAST(), NULL, NULL);
 
     return result;
 }
@@ -108,7 +110,7 @@ cleanup:
 /**
  * Perform a fast-forward merge
  *
- * @param merge_result S4 class git_merge_result
+ * @param merge_result S3 class git_merge_result
  * @param merge_head The merge head to fast-forward merge
  * @param repository The repository
  * @param log_message First part of the one line long message in the reflog
@@ -120,88 +122,99 @@ static int git2r_fast_forward_merge(
     git_repository *repository,
     const char *log_message)
 {
-    int err;
+    int error;
     const git_oid *oid;
-    git_buf buf = GIT_BUF_INIT;
+    char *buf = NULL;
+    size_t buf_len;
     git_commit *commit = NULL;
     git_tree *tree = NULL;
     git_reference *reference = NULL;
     git_checkout_options opts = GIT_CHECKOUT_OPTIONS_INIT;
 
     oid = git_annotated_commit_id(merge_head);
-    err = git_commit_lookup(&commit, repository, oid);
-    if (err)
+    error = git_commit_lookup(&commit, repository, oid);
+    if (error)
         goto cleanup;
 
-    err = git_commit_tree(&tree, commit);
-    if (err)
+    error = git_commit_tree(&tree, commit);
+    if (error)
         goto cleanup;
 
     opts.checkout_strategy = GIT_CHECKOUT_SAFE;
-    err = git_checkout_tree(repository, (git_object*)tree, &opts);
-    if (err)
+    error = git_checkout_tree(repository, (git_object*)tree, &opts);
+    if (error)
         goto cleanup;
 
-    err = git_repository_head(&reference, repository);
-    if (err) {
-        if (GIT_ENOTFOUND != err)
+    error = git_repository_head(&reference, repository);
+    if (error) {
+        if (GIT_ENOTFOUND != error)
             goto cleanup;
     }
 
-    err = git_buf_printf(&buf, "%s: Fast-forward", log_message);
-    if (err)
+    buf_len = strlen(log_message) + sizeof(": Fast-forward");
+    buf = malloc(buf_len);
+    if (!buf) {
+        GIT2R_ERROR_SET_OOM();
+        error = GIT2R_ERROR_NOMEMORY;
         goto cleanup;
+    }
+    error = snprintf(buf, buf_len, "%s: Fast-forward", log_message);
+    if (error < 0 || (size_t)error >= buf_len) {
+        GIT2R_ERROR_SET_STR(GIT2R_ERROR_OS, "Failed to snprintf log message.");
+        error = GIT2R_ERROR_OS;
+        goto cleanup;
+    }
 
-    if (GIT_ENOTFOUND == err) {
-        err = git_reference_create(
+    if (GIT_ENOTFOUND == error) {
+        error = git_reference_create(
             &reference,
             repository,
             "HEAD",
             git_commit_id(commit),
             0, /* force */
-            buf.ptr);
+            buf);
     } else {
         git_reference *target_ref = NULL;
 
-        err = git_reference_set_target(
+        error = git_reference_set_target(
             &target_ref,
             reference,
             git_commit_id(commit),
-            buf.ptr);
+            buf);
 
         if (target_ref)
             git_reference_free(target_ref);
     }
 
-    SET_SLOT(
+    SET_VECTOR_ELT(
         merge_result,
-        Rf_install("fast_forward"),
-        ScalarLogical(1));
+        git2r_S3_item__git_merge_result__fast_forward,
+        Rf_ScalarLogical(1));
 
-    SET_SLOT(
+    SET_VECTOR_ELT(
         merge_result,
-        Rf_install("conflicts"),
-        ScalarLogical(0));
+        git2r_S3_item__git_merge_result__conflicts,
+        Rf_ScalarLogical(0));
+
+    SET_VECTOR_ELT(
+        merge_result,
+        git2r_S3_item__git_merge_result__sha,
+        Rf_ScalarString(NA_STRING));
 
 cleanup:
-    git_buf_free(&buf);
+    if (buf)
+        free(buf);
+    git_commit_free(commit);
+    git_reference_free(reference);
+    git_tree_free(tree);
 
-    if (commit)
-        git_commit_free(commit);
-
-    if (reference)
-        git_reference_free(reference);
-
-    if (tree)
-        git_tree_free(tree);
-
-    return err;
+    return error;
 }
 
 /**
  * Perform a normal merge
  *
- * @param merge_result S4 class git_merge_result
+ * @param merge_result S3 class git_merge_result
  * @param merge_heads The merge heads to merge
  * @param n The number of merge heads
  * @param repository The repository
@@ -222,63 +235,90 @@ static int git2r_normal_merge(
     const git_checkout_options *checkout_opts,
     const git_merge_options *merge_opts)
 {
-    int err;
+    int error;
     git_commit *commit = NULL;
     git_index *index = NULL;
 
-    SET_SLOT(merge_result, Rf_install("fast_forward"), ScalarLogical(0));
+    SET_VECTOR_ELT(
+        merge_result,
+        git2r_S3_item__git_merge_result__fast_forward,
+        Rf_ScalarLogical(0));
 
-    err = git_merge(
+    error = git_merge(
         repository,
         merge_heads,
         n,
         merge_opts,
         checkout_opts);
-    if (err)
-        goto cleanup;
+    if (error) {
+        if (error == GIT_EMERGECONFLICT) {
+            SET_VECTOR_ELT(
+                merge_result,
+                git2r_S3_item__git_merge_result__conflicts,
+                Rf_ScalarLogical(1));
 
-    err = git_repository_index(&index, repository);
-    if (err)
+            SET_VECTOR_ELT(
+                merge_result,
+                git2r_S3_item__git_merge_result__sha,
+                Rf_ScalarString(NA_STRING));
+
+            error = 0;
+        }
+        goto cleanup;
+    }
+
+    error = git_repository_index(&index, repository);
+    if (error)
         goto cleanup;
 
     if (git_index_has_conflicts(index)) {
-        SET_SLOT(merge_result, Rf_install("conflicts"), ScalarLogical(1));
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__conflicts,
+            Rf_ScalarLogical(1));
+
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__sha,
+            Rf_ScalarString(NA_STRING));
     } else {
-        SET_SLOT(merge_result, Rf_install("conflicts"), ScalarLogical(0));
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__conflicts,
+            Rf_ScalarLogical(0));
 
         if (commit_on_success) {
             char sha[GIT_OID_HEXSZ + 1];
             git_oid oid;
-            SEXP s_sha = Rf_install("sha");
 
-            err = git2r_commit_create(
+            error = git2r_commit_create(
                 &oid,
                 repository,
                 index,
                 message,
                 merger,
                 merger);
-            if (err)
+            if (error)
                 goto cleanup;
 
             git_oid_fmt(sha, &oid);
             sha[GIT_OID_HEXSZ] = '\0';
-            SET_SLOT(merge_result, s_sha, mkString(sha));
+            SET_VECTOR_ELT(
+                merge_result,
+                git2r_S3_item__git_merge_result__sha,
+                Rf_mkString(sha));
         }
     }
 
 cleanup:
-    if (commit)
-        git_commit_free(commit);
+    git_commit_free(commit);
+    git_index_free(index);
 
-    if (index)
-        git_index_free(index);
-
-    return err;
+    return error;
 }
 
 /**
- * @param merge_result S4 class git_merge_result
+ * @param merge_result S3 class git_merge_result
  * @repository The repository
  * @param merge_head The merge head to merge
  * @param n The number of merge heads
@@ -288,6 +328,8 @@ cleanup:
  * @param merger Who is performing the merge
  * @param commit_on_success Commit merge commit, if one was created
  * during a normal merge
+ * @param fail If a conflict occurs, exit immediately instead of attempting to
+ * continue resolving conflicts.
  * @return 0 on success, or error code
  */
 static int git2r_merge(
@@ -298,9 +340,10 @@ static int git2r_merge(
     git_merge_preference_t preference,
     const char *name,
     git_signature *merger,
-    int commit_on_success)
+    int commit_on_success,
+    int fail)
 {
-    int err;
+    int error;
     git_merge_analysis_t merge_analysis;
     git_merge_preference_t merge_preference;
     git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
@@ -308,27 +351,43 @@ static int git2r_merge(
 
     merge_opts.rename_threshold = 50;
     merge_opts.target_limit = 200;
+    if (fail)
+        merge_opts.flags |= GIT_MERGE_FAIL_ON_CONFLICT;
 
     checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE;
 
-    err = git_merge_analysis(
+    error = git_merge_analysis(
         &merge_analysis,
         &merge_preference,
         repository,
         merge_heads,
         n);
-    if (err)
-        return err;
+    if (error)
+        return error;
 
     if (merge_analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE) {
-        SET_SLOT(merge_result,
-                 Rf_install("up_to_date"),
-                 ScalarLogical(1));
-        return GIT_OK;
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__up_to_date,
+            Rf_ScalarLogical(1));
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__fast_forward,
+            Rf_ScalarLogical(0));
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__conflicts,
+            Rf_ScalarLogical(0));
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__sha,
+            Rf_ScalarString(NA_STRING));
+        return 0;
     } else {
-        SET_SLOT(merge_result,
-                 Rf_install("up_to_date"),
-                 ScalarLogical(0));
+        SET_VECTOR_ELT(
+            merge_result,
+            git2r_S3_item__git_merge_result__up_to_date,
+            Rf_ScalarLogical(0));
     }
 
     if (GIT_MERGE_PREFERENCE_NONE == preference)
@@ -338,20 +397,20 @@ static int git2r_merge(
     case GIT_MERGE_PREFERENCE_NONE:
         if (merge_analysis & GIT_MERGE_ANALYSIS_FASTFORWARD) {
             if (1 != n) {
-                giterr_set_str(
-                    GITERR_NONE,
+                GIT2R_ERROR_SET_STR(
+                    GIT2R_ERROR_NONE,
                     "Unable to perform Fast-Forward merge "
                     "with mith multiple merge heads.");
                 return GIT_ERROR;
             }
 
-            err = git2r_fast_forward_merge(
+            error = git2r_fast_forward_merge(
                 merge_result,
                 merge_heads[0],
                 repository,
                 name);
         } else if (merge_analysis & GIT_MERGE_ANALYSIS_NORMAL) {
-            err = git2r_normal_merge(
+            error = git2r_normal_merge(
                 merge_result,
                 merge_heads,
                 n,
@@ -365,7 +424,7 @@ static int git2r_merge(
         break;
     case GIT_MERGE_PREFERENCE_NO_FASTFORWARD:
         if (merge_analysis & GIT_MERGE_ANALYSIS_NORMAL) {
-            err = git2r_normal_merge(
+            error = git2r_normal_merge(
                 merge_result,
                 merge_heads,
                 n,
@@ -380,25 +439,25 @@ static int git2r_merge(
     case GIT_MERGE_PREFERENCE_FASTFORWARD_ONLY:
         if (merge_analysis & GIT_MERGE_ANALYSIS_FASTFORWARD) {
             if (1 != n) {
-                giterr_set_str(
-                    GITERR_NONE,
+                GIT2R_ERROR_SET_STR(
+                    GIT2R_ERROR_NONE,
                     "Unable to perform Fast-Forward merge "
                     "with mith multiple merge heads.");
                 return GIT_ERROR;
             }
 
-            err = git2r_fast_forward_merge(
+            error = git2r_fast_forward_merge(
                 merge_result,
                 merge_heads[0],
                 repository,
                 name);
         } else {
-            giterr_set_str(GITERR_NONE, "Unable to perform Fast-Forward merge.");
+            GIT2R_ERROR_SET_STR(GIT2R_ERROR_NONE, "Unable to perform Fast-Forward merge.");
             return GIT_ERROR;
         }
         break;
     default:
-        giterr_set_str(GITERR_NONE, "Unknown merge option");
+        GIT2R_ERROR_SET_STR(GIT2R_ERROR_NONE, "Unknown merge option");
         return GIT_ERROR;
     }
 
@@ -430,18 +489,21 @@ static void git2r_merge_heads_free(git_annotated_commit **merge_heads, size_t n)
 /**
  * Merge branch into HEAD
  *
- * @param branch S4 class git_branch to merge into HEAD.
+ * @param branch S3 class git_branch to merge into HEAD.
  * @param merger Who is performing the merge
  * @param commit_on_success Commit merge commit, if one was created
  * during a normal merge
- * @return S4 class git_merge_result
+ * @param fail If a conflict occurs, exit immediately instead of attempting to
+ * continue resolving conflicts.
+ * @return S3 class git_merge_result
  */
-SEXP git2r_merge_branch(SEXP branch, SEXP merger, SEXP commit_on_success)
+SEXP git2r_merge_branch(SEXP branch, SEXP merger, SEXP commit_on_success, SEXP fail)
 {
-    int err;
+    int error, nprotect = 0;
     SEXP result = R_NilValue;
     const char *name;
-    git_buf buf = GIT_BUF_INIT;
+    char *buf = NULL;
+    size_t buf_len;
     git_branch_t type;
     git_annotated_commit **merge_heads = NULL;
     git_reference *reference = NULL;
@@ -455,68 +517,75 @@ SEXP git2r_merge_branch(SEXP branch, SEXP merger, SEXP commit_on_success)
     if (git2r_arg_check_signature(merger))
         git2r_error(__func__, NULL, "'merger'", git2r_err_signature_arg);
 
-    err = git2r_signature_from_arg(&who, merger);
-    if (err)
+    error = git2r_signature_from_arg(&who, merger);
+    if (error)
         goto cleanup;
 
-    repository = git2r_repository_open(GET_SLOT(branch, Rf_install("repo")));
+    repository = git2r_repository_open(git2r_get_list_element(branch, "repo"));
     if (!repository)
         git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
-    name = CHAR(STRING_ELT(GET_SLOT(branch, Rf_install("name")), 0));
-    type = INTEGER(GET_SLOT(branch, Rf_install("type")))[0];
-    err = git_branch_lookup(&reference, repository, name, type);
-    if (err)
+    name = CHAR(STRING_ELT(git2r_get_list_element(branch, "name"), 0));
+    type = INTEGER(git2r_get_list_element(branch, "type"))[0];
+    error = git_branch_lookup(&reference, repository, name, type);
+    if (error)
         goto cleanup;
 
     merge_heads = calloc(1, sizeof(git_annotated_commit*));
     if (NULL == merge_heads) {
-        giterr_set_str(GITERR_NONE, git2r_err_alloc_memory_buffer);
+        GIT2R_ERROR_SET_STR(GIT2R_ERROR_NONE, git2r_err_alloc_memory_buffer);
         goto cleanup;
     }
 
-    err = git_annotated_commit_from_ref(
+    error = git_annotated_commit_from_ref(
         &(merge_heads[0]),
         repository,
         reference);
-    if (err)
+    if (error)
         goto cleanup;
 
-    err = git_buf_printf(&buf, "merge %s", name);
-    if (err)
+    buf_len = strlen(name) + sizeof("merge ");
+    buf = malloc(buf_len);
+    if (!buf) {
+        GIT2R_ERROR_SET_OOM();
+        error = GIT2R_ERROR_NOMEMORY;
         goto cleanup;
+    }
+    error = snprintf(buf, buf_len, "merge %s", name);
+    if (error < 0 || (size_t)error >= buf_len) {
+        GIT2R_ERROR_SET_STR(GIT2R_ERROR_OS, "Failed to snprintf log message.");
+        error = GIT2R_ERROR_OS;
+        goto cleanup;
+    }
 
-    PROTECT(result = NEW_OBJECT(MAKE_CLASS("git_merge_result")));
-    err = git2r_merge(
+    PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_merge_result));
+    nprotect++;
+    Rf_setAttrib(result, R_ClassSymbol,
+                 Rf_mkString(git2r_S3_class__git_merge_result));
+    error = git2r_merge(
         result,
         repository,
         (const git_annotated_commit **)merge_heads,
         1,
         GIT_MERGE_PREFERENCE_NONE,
-        buf.ptr,
+        buf,
         who,
-        LOGICAL(commit_on_success)[0]);
+        LOGICAL(commit_on_success)[0],
+        LOGICAL(fail)[0]);
 
 cleanup:
-    git_buf_free(&buf);
+    if (buf)
+        free(buf);
+    git_signature_free(who);
+    git2r_merge_heads_free(merge_heads, 1);
+    git_reference_free(reference);
+    git_repository_free(repository);
 
-    if (who)
-        git_signature_free(who);
+    if (nprotect)
+        UNPROTECT(nprotect);
 
-    if (merge_heads)
-        git2r_merge_heads_free(merge_heads, 1);
-
-    if (reference)
-        git_reference_free(reference);
-
-    if (repository)
-        git_repository_free(repository);
-
-    if (!isNull(result))
-        UNPROTECT(1);
-
-    if (err)
-        git2r_error(__func__, giterr_last(), NULL, NULL);
+    if (error)
+        git2r_error(__func__, GIT2R_ERROR_LAST(), NULL, NULL);
 
     return result;
 }
@@ -527,7 +596,7 @@ cleanup:
  *
  * @param out Pointer the vector of git_annotated_commit objects.
  * @param repository The repository.
- * @param fetch_heads List of S4 class git_fetch_head objects.
+ * @param fetch_heads List of S3 class git_fetch_head objects.
  * @param n Length of fetch_heads list.
  * @return 0 on success, or error code
  */
@@ -537,58 +606,57 @@ static int git2r_merge_heads_from_fetch_heads(
     SEXP fetch_heads,
     size_t n)
 {
-    int err = GIT_OK;
+    int error = GIT_OK;
     size_t i;
 
     *merge_heads = calloc(n, sizeof(git_annotated_commit*));
     if (!(*merge_heads)) {
-        giterr_set_str(GITERR_NONE, git2r_err_alloc_memory_buffer);
+        GIT2R_ERROR_SET_STR(GIT2R_ERROR_NONE, git2r_err_alloc_memory_buffer);
         return GIT_ERROR;
     }
 
     for (i = 0; i < n; i++) {
-        int err;
         git_oid oid;
         SEXP fh = VECTOR_ELT(fetch_heads, i);
 
-        err = git_oid_fromstr(
+        error = git_oid_fromstr(
             &oid,
-            CHAR(STRING_ELT(GET_SLOT(fh, Rf_install("sha")), 0)));
-        if (err)
+            CHAR(STRING_ELT(git2r_get_list_element(fh, "sha"), 0)));
+        if (error)
             goto cleanup;
 
-        err = git_annotated_commit_from_fetchhead(
+        error = git_annotated_commit_from_fetchhead(
             &((*merge_heads)[i]),
             repository,
-            CHAR(STRING_ELT(GET_SLOT(fh, Rf_install("ref_name")), 0)),
-            CHAR(STRING_ELT(GET_SLOT(fh, Rf_install("remote_url")), 0)),
+            CHAR(STRING_ELT(git2r_get_list_element(fh, "ref_name"), 0)),
+            CHAR(STRING_ELT(git2r_get_list_element(fh, "remote_url"), 0)),
             &oid);
-        if (err)
+        if (error)
             goto cleanup;
     }
 
 cleanup:
-    if (err) {
+    if (error) {
         if (*merge_heads)
             git2r_merge_heads_free(*merge_heads, n);
         *merge_heads = NULL;
     }
 
-    return err;
+    return error;
 }
 
 /**
  * Merge the given fetch head data into HEAD
  *
- * @param fetch_heads List of S4 class git_fetch_head objects.
+ * @param fetch_heads List of S3 class git_fetch_head objects.
  * @param merger Who made the merge, if the merge is non-fast forward
  * merge that creates a merge commit.
  * @return List of git_annotated_commit objects.
  */
 SEXP git2r_merge_fetch_heads(SEXP fetch_heads, SEXP merger)
 {
-    int err;
-    size_t n;
+    int error, nprotect = 0;
+    size_t n = 0;
     SEXP result = R_NilValue;
     git_annotated_commit **merge_heads = NULL;
     git_repository *repository = NULL;
@@ -599,28 +667,32 @@ SEXP git2r_merge_fetch_heads(SEXP fetch_heads, SEXP merger)
     if (git2r_arg_check_signature(merger))
         git2r_error(__func__, NULL, "'merger'", git2r_err_signature_arg);
 
-    err = git2r_signature_from_arg(&who, merger);
-    if (err)
+    error = git2r_signature_from_arg(&who, merger);
+    if (error)
         goto cleanup;
 
     n = LENGTH(fetch_heads);
     if (n) {
-        SEXP repo = GET_SLOT(VECTOR_ELT(fetch_heads, 0), Rf_install("repo"));
+        SEXP repo = git2r_get_list_element(VECTOR_ELT(fetch_heads, 0), "repo");
         repository = git2r_repository_open(repo);
         if (!repository)
             git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
     }
 
-    err = git2r_merge_heads_from_fetch_heads(
+    error = git2r_merge_heads_from_fetch_heads(
         &merge_heads,
         repository,
         fetch_heads,
         n);
-    if (err)
+    if (error)
         goto cleanup;
 
-    PROTECT(result = NEW_OBJECT(MAKE_CLASS("git_merge_result")));
-    err = git2r_merge(
+    PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_merge_result));
+    nprotect++;
+    Rf_setAttrib(result, R_ClassSymbol,
+                 Rf_mkString(git2r_S3_class__git_merge_result));
+
+    error = git2r_merge(
         result,
         repository,
         (const git_annotated_commit **)merge_heads,
@@ -628,25 +700,19 @@ SEXP git2r_merge_fetch_heads(SEXP fetch_heads, SEXP merger)
         GIT_MERGE_PREFERENCE_NONE,
         "pull",
         who,
-        1); /* Commit on success */
-    if (err)
-        goto cleanup;
+        1,  /* Commit on success */
+        0); /* Don't fail on conflicts */
 
 cleanup:
-    if (who)
-        git_signature_free(who);
+    git_signature_free(who);
+    git2r_merge_heads_free(merge_heads, n);
+    git_repository_free(repository);
 
-    if (merge_heads)
-        git2r_merge_heads_free(merge_heads, n);
+    if (nprotect)
+        UNPROTECT(nprotect);
 
-    if (repository)
-        git_repository_free(repository);
-
-    if (!isNull(result))
-        UNPROTECT(1);
-
-    if (err)
-        git2r_error(__func__, giterr_last(), NULL, NULL);
+    if (error)
+        git2r_error(__func__, GIT2R_ERROR_LAST(), NULL, NULL);
 
     return result;
 }
