@@ -1,5 +1,5 @@
 ## git2r, R bindings to the libgit2 library.
-## Copyright (C) 2013-2015 The git2r contributors
+## Copyright (C) 2013-2019 The git2r contributors
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License, version 2,
@@ -14,23 +14,23 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-library(git2r)
+library("git2r")
 
 ## For debugging
 sessionInfo()
 
 ## Create a directory in tempdir
-path <- tempfile(pattern="git2r-")
+path <- tempfile(pattern = "git2r-")
 dir.create(path)
 
 ## Initialize a repository
 repo <- init(path)
-config(repo, user.name="Alice", user.email="alice@example.org")
+config(repo, user.name = "Alice", user.email = "alice@example.org")
 
 ## Create first commit
 writeLines("Hello world!", file.path(path, "test.txt"))
-add(repo, 'test.txt')
-commit.1 <- commit(repo, "First commit message")
+add(repo, "test.txt")
+commit_1 <- commit(repo, "First commit message")
 
 ## Edit file and checkout
 writeLines(c("Hello world!", "Hello world!"), file.path(path, "test.txt"))
@@ -62,26 +62,34 @@ stopifnot(identical(status_obs_2, status_exp_2))
 
 ## Create second commit
 writeLines(c("Hello world!", "HELLO WORLD!"), file.path(path, "test.txt"))
-add(repo, 'test.txt')
-commit.2 <- commit(repo, "Second commit message")
-tag(repo, "commit.2", "Tag message")
+add(repo, "test.txt")
+commit_2 <- commit(repo, "Second commit message")
+tag(repo, "commit_2", "Tag message")
 
 ## Create third commit
 writeLines(c("Hello world!", "HELLO WORLD!", "HeLlO wOrLd!"),
            file.path(path, "test.txt"))
-add(repo, 'test.txt')
-commit.3 <- commit(repo, "Third commit message")
+add(repo, "test.txt")
+commit_3 <- commit(repo, "Third commit message")
 
 ## Check HEAD
 stopifnot(identical(is_detached(repo), FALSE))
-stopifnot(identical(head(repo)@name, "master"))
+stopifnot(identical(repository_head(repo)$name, "master"))
+
+## Check show and summary
+repo
+summary(repo)
 
 ## Checkout first commit
-checkout(commit.1, TRUE)
+checkout(commit_1, TRUE)
 stopifnot(identical(is_detached(repo), TRUE))
-stopifnot(identical(head(repo), commit.1))
+stopifnot(identical(repository_head(repo), commit_1))
 stopifnot(identical(readLines(file.path(path, "test.txt")), "Hello world!"))
 
+## Check show and summary
+repo
+summary(repo)
+
 ## Checkout tag
 checkout(tags(repo)[[1]], TRUE)
 stopifnot(identical(is_detached(repo), TRUE))
@@ -95,4 +103,4 @@ if (!is.null(wd))
     setwd(wd)
 
 ## Cleanup
-unlink(path, recursive=TRUE)
+unlink(path, recursive = TRUE)
