@@ -1,5 +1,5 @@
 ## git2r, R bindings to the libgit2 library.
-## Copyright (C) 2013-2015 The git2r contributors
+## Copyright (C) 2013-2019 The git2r contributors
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License, version 2,
@@ -14,18 +14,18 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-library(git2r)
+library("git2r")
 
 ## For debugging
 sessionInfo()
 
 ## Initialize a temporary repository
-path <- tempfile(pattern="git2r-")
+path <- tempfile(pattern = "git2r-")
 dir.create(path)
 repo <- init(path)
 
 ## Create a user and commit a file
-config(repo, user.name="Author", user.email="author@example.org")
+config(repo, user.name = "Author", user.email = "author@example.org")
 writeLines(c("First line in file 1.", "Second line in file 1."),
            file.path(path, "example-1.txt"))
 add(repo, "example-1.txt")
@@ -54,18 +54,27 @@ writeLines(c("First line in file 2.", "line Second in file 2."),
 add(repo, "example-2.txt")
 commit(repo, "Fourth commit message")
 
+# Missing branch to merge with should throw an error
+tools::assertError(merge(repo))
+
 ## Merge 'fix'
-git2r:::merge_named_branch(repo, "fix", TRUE, default_signature(repo))
+m <- merge(repo, "fix", TRUE, default_signature(repo))
+stopifnot(identical(format(m), "Merge"))
+
+## Merge 'fix' again
+m <- merge(repo, "fix", TRUE, default_signature(repo))
+stopifnot(identical(format(m), "Already up-to-date"))
 
 ## Check number of parents of each commit
 stopifnot(identical(sapply(commits(repo), function(x) length(parents(x))),
                     c(2L, 1L, 1L, 1L, 0L)))
 
 ## Check that last commit is a merge
-stopifnot(is_merge(lookup(repo, branch_target(head(repo)))))
+stopifnot(is_merge(last_commit(repo)))
+summary(last_commit(repo))
 
 ## Check that metadata associated with merge is removed
 stopifnot(!file.exists(file.path(path, ".git", "MERGE_HEAD")))
 
 ## Cleanup
-unlink(path, recursive=TRUE)
+unlink(path, recursive = TRUE)
