@@ -1,5 +1,5 @@
 ## git2r, R bindings to the libgit2 library.
-## Copyright (C) 2013-2018 The git2r contributors
+## Copyright (C) 2013-2019 The git2r contributors
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License, version 2,
@@ -20,9 +20,9 @@ library("git2r")
 sessionInfo()
 
 ## Create directories for repositories in tempdir
-path_bare <- tempfile(pattern="git2r-")
-path_repo_1 <- tempfile(pattern="git2r-")
-path_repo_2 <- tempfile(pattern="git2r-")
+path_bare <- tempfile(pattern = "git2r-")
+path_repo_1 <- tempfile(pattern = "git2r-")
+path_repo_2 <- tempfile(pattern = "git2r-")
 
 dir.create(path_bare)
 dir.create(path_repo_1)
@@ -33,7 +33,7 @@ bare_repo <- init(path_bare, bare = TRUE)
 
 ## Clone to repo 1
 repo_1 <- clone(path_bare, path_repo_1)
-config(repo_1, user.name="Alice", user.email="alice@example.org")
+config(repo_1, user.name = "Alice", user.email = "alice@example.org")
 
 ## Add changes to repo 1 and push to bare
 writeLines("Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do",
@@ -44,7 +44,7 @@ push(repo_1, "origin", "refs/heads/master")
 
 ## Clone to repo 2
 repo_2 <- clone(path_bare, path_repo_2)
-config(repo_2, user.name="Bob", user.email="bob@example.org")
+config(repo_2, user.name = "Bob", user.email = "bob@example.org")
 
 ## Add more changes to repo 1 and push to bare
 writeLines(c("Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do",
@@ -59,8 +59,9 @@ pull(repo_2)
 stopifnot(identical(length(commits(repo_2)), 2L))
 
 ## Check remote url of repo_2
-stopifnot(identical(branch_remote_url(branch_get_upstream(repository_head(repo_2))),
-                    path_bare))
+stopifnot(identical(
+    branch_remote_url(branch_get_upstream(repository_head(repo_2))),
+    path_bare))
 
 ## Unset remote remote tracking branch
 branch_set_upstream(repository_head(repo_2), NULL)
@@ -69,26 +70,35 @@ tools::assertError(pull(repo_2))
 tools::assertError(branch_set_upstream(repository_head(repo_2), NULL))
 
 ## Add more changes to repo 1 and push to bare
-writeLines(c("Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do",
-             "eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad",
-             "minim veniam, quis nostrud exercitation ullamco laboris nisi ut"),
-           con = file.path(path_repo_1, "test-1.txt"))
+writeLines(
+    c("Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do",
+      "eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad",
+      "minim veniam, quis nostrud exercitation ullamco laboris nisi ut"),
+    con = file.path(path_repo_1, "test-1.txt"))
 add(repo_1, "test-1.txt")
 commit_3 <- commit(repo_1, "Third commit message")
 push(repo_1)
 
 ## Set remote tracking branch
 branch_set_upstream(repository_head(repo_2), "origin/master")
-stopifnot(identical(branch_remote_url(branch_get_upstream(repository_head(repo_2))),
-                    path_bare))
+stopifnot(identical(
+    branch_remote_url(branch_get_upstream(repository_head(repo_2))),
+    path_bare))
 
 ## Pull changes to repo_2
 pull(repo_2)
 stopifnot(identical(length(commits(repo_2)), 3L))
 
-## Check references in repo_1 and repo_2
+## Check references in repo_1 and repo_2. Must clear the repo item
+## since the repositories have different paths.
 stopifnot(identical(length(references(repo_1)), 2L))
-stopifnot(identical(references(repo_1), references(repo_2)))
+ref_1 <- references(repo_1)
+ref_1[[1]]$repo <- NULL
+ref_1[[2]]$repo <- NULL
+ref_2 <- references(repo_2)
+ref_2[[1]]$repo <- NULL
+ref_2[[2]]$repo <- NULL
+stopifnot(identical(ref_1, ref_2))
 
 ref_1 <- references(repo_1)[["refs/heads/master"]]
 stopifnot(identical(ref_1$name, "refs/heads/master"))
@@ -111,6 +121,6 @@ if (!is.null(wd))
     setwd(wd)
 
 ## Cleanup
-unlink(path_bare, recursive=TRUE)
-unlink(path_repo_1, recursive=TRUE)
-unlink(path_repo_2, recursive=TRUE)
+unlink(path_bare, recursive = TRUE)
+unlink(path_repo_1, recursive = TRUE)
+unlink(path_repo_2, recursive = TRUE)
